#!/usr/bin/env python3
#coding=utf8

from optparse import OptionParser
import shutil
import os
import subprocess

svg2eps = 'inkscape %s -C --export-filename=%s.eps --export-type=eps --export-latex'
svg2pdf = 'inkscape %s -C --export-filename=%s.pdf --export-type=pdf --export-latex'

def create_image(input_filename, output_filename, mode, ext):
    subprocess.Popen(mode % (input_filename, output_filename),
        stdout=subprocess.PIPE, shell=True).communicate()[0]

    o_ext = output_filename + '.' + ext
    o = output_filename
    o_ext_tex = output_filename + '.' + ext + '_tex'
    o_t = output_filename + '_t'

    shutil.move(o_ext, o)

    fin = open(o_ext_tex, 'r')
    fout = open(o_t, 'w')

    #\includegraphics[width=\unitlength,page=1]{logo.pdftex}
    for line in fin:
        # FIXME: be more conservative in the replacement
        # (in case '{'+o_ext+'}' appeares somewhere else)
        out = line.replace('{'+os.path.basename(o_ext)+'}', '{'+os.path.basename(o)+'}')
        fout.write(out)

    fin.close()
    fout.close()
    os.remove(o_ext_tex)

def main():
    parser = OptionParser()
    parser.add_option("-L", "--format", dest="outputFormat",
        metavar="FORMAT", help="output format", default="spstex")
    parser.add_option("-p", "--portrait", dest="portrait", help="dummy arg")
    (options, args) = parser.parse_args()
    if len(args) != 2: return
    (input_filename, output_filename) = args
    fmt = options.outputFormat
    portrait = options.portrait

    if fmt == 'eps':
        create_image(input_filename, output_filename, svg2eps, 'eps')
    elif fmt == 'spstex' or fmt == 'pstex':
        create_image(input_filename, output_filename, svg2eps, 'eps')
    elif fmt == 'spstex_t' or fmt == 'pstex_t':
        pass
    elif fmt == 'spdftex' or fmt == 'pdftex':
        create_image(input_filename, output_filename, svg2pdf, 'pdf')
    elif fmt == 'spdftex_t' or fmt == 'pdftex_t':
        pass

if __name__ == "__main__":
    main()




# PuyoTikZ package for LaTeX.
# Created by amosborne (GitHub) / terramyst (Twitter).
# MIT License

import numpy as np
import itertools
import string
import re


# Wraps a function which returns a string. Prints the result with a ";".
def print_tikz(func):
    def wrapper(*args, **kwargs):
        print(func(*args, **kwargs) + ";")

    return wrapper


PUYO_RADIUS, PUYO_JOIN = 0.45, 0.25
COLORS = {
    "r": "red",
    "y": "yellow",
    "g": "green",
    "b": "blue",
    "p": "purple",
    "n": "gray",
}
for key, value in COLORS.items():
    COLORS[key] = value + "puyo"


def puyoboard(cols, rows, hrows, boardpuyos, nextpuyos, label):
    if not cols > 0:
        raise UserWarning("{0} columns must be atleast 1.".format(cols))
    if not rows > 0:
        raise UserWarning("{0} rows must be atleast 1.".format(rows))
    if hrows < 0:
        raise UserWarning("{0} hidden rows must be atleast 0.".format(hrows))

    draw_board(cols, rows, hrows, boardpuyos)
    draw_nextpuyos(cols, rows, nextpuyos)
    if label:
        draw_labels(cols, rows, hrows, nextpuyos)


def puyomarker(config):
    pattern = "^([a-z]+)(\d+)([rygbpn])([A-Z])$"

    @print_tikz
    def singlemark(subconfig):
        match = re.search(pattern, subconfig)
        if match is None:
            raise UserWarning("Bad puyo marker format string.")
        else:
            colid, rowid, puyo, lid = match.groups()

        rowidx = int(rowid) - 1
        colidx = next(
            idx for idx, x in enumerate(excel_cols(upper=False)) if x == colid
        )
        pos = colidx + 0.5, rowidx + 0.5

        tikz = "\\draw[{0},thick]".format("dark" + COLORS[puyo])
        tikz += " ({0},{1}) circle ({2});".format(*pos, PUYO_RADIUS)
        tikz += " \\node[anchor=center, font=\sffamily]"
        tikz += " at ({0},{1}) {{\\small {2}}}".format(*pos, lid)
        return tikz

    subconfigs = config.split("/")
    if subconfigs == [""]:
        return
    else:
        for subconfig in subconfigs:
            singlemark(subconfig)


def draw_board(cols, rows, hrows, boardpuyos):
    # draw the board grid
    draw_grid(params=["gray", "ultra thin"], size=(cols, rows))
    draw_grid(
        params=["gray", "ultra thin", "line cap=round"],
        origin=(0, rows),
        size=(cols, hrows),
    )
    draw_grid(
        params=["black", "line cap=round"], stepsize=(cols, rows), size=(cols, rows)
    )

    # draw the puyos on the board
    draw_puyos(puyos=boardpuyos.split("/"), size=(cols, rows), nhidden=hrows)


def draw_nextpuyos(cols, rows, nextpuyos):
    nextpuyos = nextpuyos.split("/")
    if nextpuyos == [""]:
        return
    for idx, puyos in enumerate(nextpuyos):
        if not len(puyos) == 2:
            raise UserWarning("Next puyos must be of length 2.")
        draw_puyos(
            puyos=[puyos], origin=(cols + 0.75, rows - 1.5 - 3 * idx), size=(1, 2)
        )


def draw_labels(cols, rows, hrows, nextpuyos):
    @print_tikz
    def draw_label(pos, text, size="\\normalsize", style="\\rmfamily"):
        return "\\node[anchor=mid, font={4}] at ({0},{1}) {{{3} {2}}}".format(
            *pos, text, size, style
        )

    for idx in range(0, rows + hrows):
        draw_label((-0.5, 0.5 + idx), idx + 1)

    for idx, label in zip(range(0, cols), excel_cols(upper=False)):
        draw_label((0.5 + idx, -0.5), label)

    nextpuyoscount = len(nextpuyos.split("/"))
    if nextpuyos.split("/") == [""]:
        return
    for idx, label in zip(range(0, nextpuyoscount), excel_cols(upper=True)):
        draw_label((cols + 0.75, rows - 1.5 - 3 * idx), label, "\\small", "\\sffamily")


@print_tikz
def draw_grid(params=[], origin=(0, 0), stepsize=(1, 1), size=(6, 12)):
    extent = tuple([x + y for x, y in zip(origin, size)])
    tikz = "\\draw"
    tikz += "[" + ",".join(params) + "]"
    tikz += " ({0},{1})".format(*origin)
    tikz += " grid [xstep={0},ystep={1}]".format(*stepsize)
    tikz += " ({0},{1})".format(*extent)
    return tikz


def draw_puyos(puyos=[], origin=(0, 0), size=(6, 12), nhidden=0):
    # construct a 2D matrix of strings (representing puyos)
    board = np.full((size[0], size[1] + nhidden), "")
    for ridx, col in enumerate(puyos):
        for cidx, puyo in enumerate(list(col)):
            if puyo not in COLORS.keys():
                raise UserWarning(
                    "{0} is not a valid puyo identifier (rygbpn).".format(puyo)
                )
            try:
                board[ridx, cidx] = puyo
            except IndexError:
                raise UserWarning("Puyo layout string has too many rows or columns.")

    # helper function for converting true position
    def true_pos(pos):
        return tuple([x + y + 0.5 for x, y in zip(pos, origin)])

    # draw all the puyo circles
    for pos, puyo in np.ndenumerate(board):
        if puyo:
            draw_puyo(true_pos(pos), puyo)

    # connect to adjacent puyos (except nuisance and hidden row)
    for pos, puyo in np.ndenumerate(board):
        for direc in {(1, 0), (-1, 0), (0, 1), (0, -1)}:
            if not 0 <= pos[0] + direc[0] < board.shape[0]:
                continue
            elif not 0 <= pos[1] + direc[1] < board.shape[1] - nhidden:
                continue
            elif not 0 <= pos[1] < board.shape[1] - nhidden:
                continue

            other = board[pos[0] + direc[0], pos[1] + direc[1]]
            if puyo and other == puyo and puyo != "n":
                connect_puyo(true_pos(pos), direc, puyo)

    # draw all the puyo faces
    for pos, puyo in np.ndenumerate(board):
        if puyo:
            draw_face(true_pos(pos), puyo)

    # outline the puyos with a darker color
    for pos, puyo in np.ndenumerate(board):
        for direc in {(1, 0), (-1, 0), (0, 1), (0, -1)}:
            if not 0 <= pos[0] + direc[0] < board.shape[0]:
                if puyo:
                    outline_puyo(true_pos(pos), direc, puyo, join=False)
                continue
            elif not 0 <= pos[1] + direc[1] < board.shape[1] - nhidden:
                if puyo:
                    outline_puyo(true_pos(pos), direc, puyo, join=False)
                continue
            elif not 0 <= pos[1] < board.shape[1] - nhidden:
                if puyo:
                    outline_puyo(true_pos(pos), direc, puyo, join=False)
                continue

            other = board[pos[0] + direc[0], pos[1] + direc[1]]
            if puyo and other == puyo and puyo != "n":
                outline_puyo(true_pos(pos), direc, puyo, join=True)
            elif puyo:
                outline_puyo(true_pos(pos), direc, puyo, join=False)


@print_tikz
def draw_face(pos=(0, 0), puyo="r"):
    def tikz_shift(shift=(0, 0)):
        total_shift = pos[0] + shift[0], pos[1] + shift[1]
        return "[xshift={0}cm,yshift={1}cm]".format(*total_shift)

    tikz = "\\draw[white,semithick]"
    if puyo == "p":
        tikz += " ({0}150:{1}) -- ({0}-30:{1});".format(tikz_shift(), PUYO_RADIUS)
    elif puyo == "r":
        tikz += " ({0}150:{1}) -- ({0}0,0) -- ({0}30:{1});".format(
            tikz_shift(), PUYO_RADIUS
        )
    elif puyo == "g":
        tikz += " ({0}-90:{1}) arc (-90:0:{1})".format(
            tikz_shift((-PUYO_RADIUS, PUYO_RADIUS)), PUYO_RADIUS
        )
        tikz += " ({0}-90:{1}) arc (-90:-180:{1});".format(
            tikz_shift((PUYO_RADIUS, PUYO_RADIUS)), PUYO_RADIUS
        )
    elif puyo == "b":
        tikz += " ({0}180:{1}) -- ({0}90:{2}) -- ({0}0:{1});".format(
            tikz_shift(), PUYO_RADIUS, PUYO_RADIUS * np.sin(np.deg2rad(30))
        )
    elif puyo == "y":
        tikz += " ({0}150:{1}) -- ({0}30:{1});".format(tikz_shift(), PUYO_RADIUS)
    elif puyo == "n":
        tikz += " ({0}210:{1}) -- ({0}-30:{1});".format(tikz_shift(), PUYO_RADIUS)
    else:
        tikz = ""

    return tikz


@print_tikz
def draw_puyo(pos=(0, 0), puyo="r"):
    tikz = "\\draw[{0},fill={0}]".format(COLORS[puyo])
    tikz += " ({0},{1}) circle ({2})".format(*pos, PUYO_RADIUS)
    return tikz


@print_tikz
def connect_puyo(pos=(0, 0), direc=(1, 0), puyo="r"):
    phi, join1, join2 = connect_puyo_params(pos, direc)
    tikz = "\\draw[{0},fill={0}]".format(COLORS[puyo])
    tikz += " ([xshift={0}cm,yshift={1}cm]".format(*pos)
    tikz += "{0}:{2}) arc ({0}:{1}:{2})".format(phi - 45, phi + 45, PUYO_RADIUS)
    tikz += " to[out={0},in={1}] ({2},{3})".format(phi - 45, phi - 180, *join1)
    tikz += " -- ({0},{1})".format(*join2)
    tikz += " to[out={0},in={1}] cycle".format(phi - 180, phi + 45)
    return tikz


@print_tikz
def outline_puyo(pos=(0, 0), direc=(1, 0), puyo="r", join=False):
    phi, join1, join2 = connect_puyo_params(pos, direc)
    tikz = "\\draw[{0}]".format("dark" + COLORS[puyo])

    if join:
        tikz += " ([xshift={0}cm,yshift={1}cm]".format(*pos)
        tikz += "{0}:{1})".format(phi + 45, PUYO_RADIUS)
        tikz += " to[out={0},in={1}] ({2},{3})".format(phi - 45, phi - 180, *join1)
        tikz += " ({0},{1})".format(*join2)
        tikz += " to[out={0},in={1}]".format(phi - 180, phi + 45)
        tikz += " ([xshift={0}cm,yshift={1}cm]".format(*pos)
        tikz += "{0}:{1})".format(phi - 45, PUYO_RADIUS)
    else:
        tikz += " ([xshift={0}cm,yshift={1}cm]".format(*pos)
        tikz += "{0}:{2}) arc ({0}:{1}:{2})".format(phi - 45, phi + 45, PUYO_RADIUS)

    return tikz


def connect_puyo_params(pos, direc):
    """Determine the path angle and midpoint locations."""
    phi = np.rad2deg(np.arctan2(direc[1], direc[0])) % 360
    x, y1, y2 = 0.5, PUYO_JOIN, -PUYO_JOIN

    c, s = np.cos(np.deg2rad(phi)), np.sin(np.deg2rad(-phi))
    j = np.matrix([[c, s], [-s, c]])

    def rotate_point(y):
        m = np.dot(j, [x, y])
        return float(m.T[0]) + pos[0], float(m.T[1]) + pos[1]

    return phi, rotate_point(y1), rotate_point(y2)


def excel_cols(upper=True):
    n = 1
    letters = string.ascii_uppercase if upper else string.ascii_lowercase
    while True:
        yield from ("".join(group) for group in itertools.product(letters, repeat=n))
        n += 1



#!/usr/bin/env python3
#coding=utf8

"""

stdin : the original xfig file
stdout : the output xfig file
args : all depths we want to keep

"""

from __future__ import print_function
import optparse
import os.path
import sys

def main():
    parser = optparse.OptionParser()
    (options, args) = parser.parse_args()

    depths_to_keep = set()
    for arg in args:
        depths_to_keep.add(arg)

    comment = ''
    display = True
    def show(depth, line):
        if depth in depths_to_keep:
            print(comment+line, end='')
            return True
        else:
            return False
    for line in sys.stdin:
        if line[0] == '#':
            comment += line
            continue
        if line[0] in "\t ":
            if display:
                print(line, end='')
        else:
            Fld = line.split(' ', 9999)
            if not Fld[0] or Fld[0] not in ('1', '2', '3', '4', '5'):
                print(comment+line, end='')
                display = True
            elif Fld[0] == '4':
                display = show(Fld[3], line)
            else:
                display = show(Fld[6], line)
            comment = ''

if __name__ == "__main__":
    main()



#!/usr/bin/env python3
#
# This file is a part of Memoize, a TeX package for externalization of
# graphics and memoization of compilation results in general, available at
# https://ctan.org/pkg/memoize and https://github.com/sasozivanovic/memoize.
#
# Copyright (c) 2020- Saso Zivanovic <saso.zivanovic@guest.arnes.si>
#
# This work may be distributed and/or modified under the conditions of the
# LaTeX Project Public License, either version 1.3c of this license or (at
# your option) any later version.  The latest version of this license is in
# https://www.latex-project.org/lppl.txt and version 1.3c or later is part of
# all distributions of LaTeX version 2008 or later.
#
# This work has the LPPL maintenance status `maintained'.
# The Current Maintainer of this work is Saso Zivanovic.
# 
# The files belonging to this work and covered by LPPL are listed in
# <texmf>/doc/generic/memoize/FILES.

__version__ = '2024/12/02 v1.4.1'

import argparse, re, sys, os, subprocess, itertools, traceback, platform
from pathlib import Path, PurePath
# We will only try to import the PDF processing library once we set up the
# error log.

# \paragraph{Messages}

# The messages are written both to the extraction log and the terminal (we
# output to stdout rather than stderr so that messages on the TeX terminal and
# document |.log| appear in chronological order).  Messages are automatically
# adapted to the TeX |--format|.

# The format of the messages.  It depends on the given |--format|; the last
# entry is for t the terminal output.

ERROR = {
    'latex':   r'\PackageError{{{package_name}}}{{{short}}}{{{long}}}',
    'plain':   r'\errhelp{{{long}}}\errmessage{{{package_name}: {short}}}',
    'context': r'\errhelp{{{long}}}\errmessage{{{package_name}: {short}}}',
    None:      '{header}{short}.\n{long}',
}

WARNING = {
    'latex':   r'\PackageWarning{{{package_name}}}{{{texindent}{text}}}',
    'plain':   r'\message{{{package_name}: {texindent}{text}}}',
    'context': r'\message{{{package_name}: {texindent}{text}}}',
    None:      r'{header}{indent}{text}.',
}

INFO = {
    'latex':   r'\PackageInfo{{{package_name}}}{{{texindent}{text}}}',
    'plain':   r'\message{{{package_name}: {texindent}{text}}}',
    'context': r'\message{{{package_name}: {texindent}{text}}}',
    None:      r'{header}{indent}{text}.',    
}

# Some variables used in the message routines; note that |header| will be
# redefined once we parse the arguments.

package_name = 'memoize (python-based extraction)'
exit_code = 0
log = None
header = ''
indent = ''
texindent = ''

# The message routines.

def error(short, long):
    if not args.quiet:
        print(ERROR[None].format(short = short, long = long, header = header))
    if log:
        short = short.replace('\\', '\\string\\')
        long = long.replace('\\', '\\string\\')
        print(
            ERROR[args.format].format(
                short = short, long = long, package_name = package_name),
            file = log)
    global exit_code
    exit_code = 11
    endinput()
    
def warning(text):
    if text and not args.quiet:
        print(WARNING[None].format(text = text, header = header, indent = indent))
    if log:
        text = text.replace('\\', '\\string\\')
        print(
            WARNING[args.format].format(
                text = text, texindent = texindent, package_name = package_name),
            file = log)
    global exit_code
    exit_code = 10
    
def info(text):
    if text and not args.quiet:
        print(INFO[None].format(text = text, header = header, indent = indent))
        if log:
            text = text.replace('\\', '\\string\\')
            print(
                INFO[args.format].format(
                    text = text, texindent = texindent, package_name = package_name),
                file = log)
            
# Mark the log as complete and exit.
def endinput():
    if log:
        print(r'\endinput', file = log)
        log.close()
    sys.exit(exit_code)

# \paragraph{Permission-related functions}

# |paranoia_in|/|out| should work exactly as
# |kpsewhich -safe-in-name|/|-safe-out-name|.
def paranoia_in(f, remark = ''):
    if f and not _paranoia(f, openin_any):
        error(f"I'm not allowed to read from '{f}' (openin_any = {openin_any})",
              remark)

def paranoia_out(f, remark = ''):
    if f and not _paranoia(f, openout_any):
        error(f"I'm not allowed to write to '{f}' (openout_any = {openout_any})",
              remark)

def _paranoia(f, mode):
    # |mode| is the value of |openin_any| or |openout_any|.  |f| is a
    # |pathlib.Path| object.
    return (
        # In mode `any' (|a|, |y| or |1|), we may access any file.
        mode in 'ay1'
        or (
        # Otherwise, we are at least in the restricted mode, so we should not
        # open dot files on Unix-like systems (except file called |.tex|).
            not (os.name == 'posix' and f.stem.startswith('.') and f.stem != '.tex')
            and (
                # If we are precisely in the restricted mode (|r|, |n|, |0|),
                # then there are no further restrictions.
                mode in 'rn0'
                # Otherwise, we are in the paranoid mode (officially |p|, but
                # any other value is interpreted as |p| as well).  There are
                # two further restrictions in the paranoid mode.
                or (
                    # We're not allowed to go to a parent directory.
                    '..' not in f.parts
                    and
                    # If the given path is absolute, is should be a descendant
                    # of either |TEXMF_OUTPUT_DIRECTORY| or |TEXMFOUTPUT|.
                    (not f.is_absolute()
                     or
                     is_ancestor(texmf_output_directory, f)
                     or
                     is_ancestor(texmfoutput, f)
                     )))))

# On Windows, we disallow ``semi-absolute'' paths, i.e.\ paths starting with
# the |\| but lacking the drive.  On Windows, |pathlib|'s |is_absolute| returns
# |True| only for paths starting with |\| and containing the drive.
def sanitize_filename(f):
    if f and platform.system() == 'Windows' and not (f.is_absolute() or not f.drive):
        error(f"\"Semi-absolute\" paths are disallowed: '{f}'", r"The path must "
              r"either contain both the drive letter and start with '\', "
              r"or none of these; paths like 'C:foo' and '\foo' are disallowed")

def access_in(f):
    return os.access(f, os.R_OK)

# This function can fail on Windows, reporting a non-writable file or dir as
# writable, because |os.access| does not work with Windows' |icacls|
# permissions.  Consequence: we might try to write to a read-only current or
# output directory instead of switching to the temporary directory.  Paranoia
# is unaffected, as it doesn't use |access_*| functions.
def access_out(f):
    try:
        exists = f.exists()
    # Presumably, we get this error when the parent directory is not
    # executable.
    except PermissionError:
        return
    if exists:
        # An existing file should be writable, and if it's a directory, it
        # should also be executable.
        return os.access(f, os.W_OK) and (not f.is_dir() or os.access(f, os.X_OK))
    else:
        # For a non-existing file, the parent directory should be writable.
        # (This is the only place where function |pathlib.parent| is used, so
        # it's ok that it returns the logical parent.)
        return os.access(f.parent, os.W_OK)

# This function finds the location for an input file, respecting
# |TEXMF_OUTPUT_DIRECTORY| and |TEXMFOUTPUT|, and the permissions in the
# filesystem.  It returns an absolute file as-is.  For a relative file, it
# tries |TEXMF_OUTPUT_DIRECTORY| (if defined), the current directory (always),
# and |TEXMFOUTPUT| directory (if defined), in this order.  The first readable
# file found is returned; if no readable file is found, the file in the current
# directory is returned.
def find_in(f):
    sanitize_filename(f)
    if f.is_absolute():
        return f
    for df in (texmf_output_directory / f if texmf_output_directory else None,
               f,
               texmfoutput / f if texmfoutput else None):
        if df and access_in(df):
            return df
    return f

# This function finds the location for an output file, respecting
# |TEXMF_OUTPUT_DIRECTORY| and |TEXMFOUTPUT|, and the permissions in the
# filesystem.  It returns an absolute file as-is.  For a relative file, it
# tries |TEXMF_OUTPUT_DIRECTORY| (if defined), the current directory (unless
# |TEXMF_OUTPUT_DIRECTORY| is defined), and |TEXMFOUTPUT| directory (if
# defined), in this order.  The first writable file found is returned; if no
# writable file is found, the file in either the current or the output
# directory is returned.
def find_out(f):
    sanitize_filename(f)
    if f.is_absolute():
        return f
    for df in (texmf_output_directory / f if texmf_output_directory else None,
               f if not texmf_output_directory else None,
               texmfoutput / f if texmfoutput else None):
        if df and access_out(df):
            return df
    return texmf_output_directory / f if texmf_output_directory else f

# This function assumes that both paths are absolute; ancestor may be |None|,
# signaling a non-path.
def is_ancestor(ancestor, descendant):
    if not ancestor:
        return
    a = ancestor.parts
    d = descendant.parts
    return len(a) < len(d) and a == d[0:len(a)]

# A paranoid |Path.mkdir|.  The given folder is preprocessed by |find_out|.
def mkdir(folder):
    folder = find_out(Path(folder))
    if not folder.exists():
        paranoia_out(folder)
        # Using |folder.mkdir| is fine because we know that
        # |TEXMF_OUTPUT_DIRECTORY|/|TEXMFOUTPUT|, if given, exists, and that
        # ``folder'' contains no |..|.
        folder.mkdir(parents = True, exist_ok = True)
        # This does not get logged when the function is invoked via |--mkdir|,
        # as it is not clear what the log name should be.
        info(f"Created directory {folder}")

_re_unquote = re.compile(r'"(.*?)"')
def unquote(fn):
    return _re_unquote.sub(r'\1', fn)
        
# \paragraph{Kpathsea}

# Get the values of |openin_any|, |openout_any|, |TEXMFOUTPUT| and
# |TEXMF_OUTPUT_DIRECTORY|.

kpsewhich_output = subprocess.run(['kpsewhich',
                                   f'-expand-var='
                                   f'openin_any=$openin_any,'
                                   f'openout_any=$openout_any,'
                                   f'TEXMFOUTPUT=$TEXMFOUTPUT'],
                                  capture_output = True
                                  ).stdout.decode().strip()
if not kpsewhich_output:
    # No TeX? (Note that |kpsewhich| should exist in MiKTeX as well.)  In
    # absence of |kpathsea| information, we get very paranoid, but still try to
    # get |TEXMFOUTPUT| from an environment variable.
    openin_any, openout_any = 'p', 'p'
    texmfoutput, texmf_output_directory = None, None
    # Unfortunately, this warning can't make it into the log.  But then again,
    # the chances of a missing |kpsewhich| are very slim, and its absence would
    # show all over the place anyway.
    warning('I failed to execute "kpsewhich"; , is there no TeX system installed? '
            'Assuming openin_any = openout_any = "p" '
            '(i.e. restricting all file operations to non-hidden files '
            'in the current directory of its subdirectories).')
else:
    m = re.fullmatch(r'openin_any=(.*),openout_any=(.*),TEXMFOUTPUT=(.*)',
                     kpsewhich_output)
    openin_any, openout_any, texmfoutput = m.groups()
    texmf_output_directory = os.environ.get('TEXMF_OUTPUT_DIRECTORY', None)
    if openin_any == '$openin_any':
        # When the |open*_any| variables are not expanded, we assume we're
        # running MiKTeX. The two config settings below correspond to TeXLive's
        # |openin_any| and |openout_any|; afaik, there is no analogue to
        # |TEXMFOUTPUT|.
        initexmf_output = subprocess.run(
            ['initexmf', '--show-config-value=[Core]AllowUnsafeInputFiles',
             '--show-config-value=[Core]AllowUnsafeOutputFiles'],
            capture_output = True).stdout.decode().strip()
        openin_any, openout_any = initexmf_output.split()
        openin_any = 'a' if openin_any == 'true' else 'p'
        openout_any = 'a' if openout_any == 'true' else 'p'
        texmfoutput = None
        texmf_output_directory = None

# An output directory should exist, and may not point to the root on Linux. On
# Windows, it may point to the root, because we only allow absolute filenames
# containing the drive, e.g.\ |F:\|; see |is_absolute|.
def sanitize_output_dir(d_str):
    d = Path(d_str) if d_str else None
    sanitize_filename(d)
    return d if d and d.is_dir() and \
        (not d.is_absolute() or len(d.parts) != 1 or d.drive) else None

texmfoutput = sanitize_output_dir(texmfoutput)
texmf_output_directory = sanitize_output_dir(texmf_output_directory)

class NotExtracted(UserWarning):
    pass

# We don't delve into the real script when loaded from the testing code.
if __name__ == '__main__':

    # \paragraph{Arguments}

    parser = argparse.ArgumentParser(
        description = "Extract extern pages produced by package Memoize "
                      "out of the document PDF.",
        epilog = "For details, see the man page or the Memoize documentation.",
        prog = 'memoize-extract.py',
    )
    parser.add_argument('-P', '--pdf', help = 'extract from file PDF')
    parser.add_argument('-p', '--prune', action = 'store_true',
        help = 'remove the extern pages after extraction')
    parser.add_argument('-k', '--keep', action = 'store_true',
        help = 'do not mark externs as extracted')
    parser.add_argument('-F', '--format', choices = ['latex', 'plain', 'context'],
        help = 'the format of the TeX document invoking extraction')
    parser.add_argument('-f', '--force', action = 'store_true',
        help = 'extract even if the size-check fails')
    parser.add_argument('-q', '--quiet', action = 'store_true',
        help = "describe what's happening")
    parser.add_argument('-m', '--mkdir', action = 'store_true',
        help = 'create a directory (and exit); '
               'mmz argument is interpreted as directory name')
    parser.add_argument('-V', '--version', action = 'version',
        version = f"%(prog)s of Memoize " + __version__)
    parser.add_argument('mmz', help = 'the record file produced by Memoize: '
                                      'doc.mmz when compiling doc.tex '
                                      '(doc and doc.tex are accepted as well)')

    args = parser.parse_args()

    header = parser.prog + ': ' if args.format else ''
    
    # Start a new line in the TeX terminal output.
    if args.format:
        print()

    # \paragraph{Initialization}

    # With |--mkdir|, argument |mmz| is interpreted as the directory to create.
    if args.mkdir:
        mkdir(args.mmz)
        sys.exit()

    # Normalize the |mmz| argument into a |.mmz| filename.
    mmz_file = Path(args.mmz)
    if mmz_file.suffix == '.tex':
        mmz_file = mmz_file.with_suffix('.mmz')
    elif mmz_file.suffix != '.mmz':
        mmz_file = mmz_file.with_name(mmz_file.name + '.mmz')

    # Once we have the |.mmz| filename, we can open the log.
    if args.format:
        log_file = find_out(mmz_file.with_suffix('.mmz.log'))
        paranoia_out(log_file)
        info(f"Logging to '{log_file}'");
        log = open(log_file, 'w')

    # Now that we have opened the log file, we can try loading the PDF
    # processing library.
    try:
        import pdfrw
    except ModuleNotFoundError:
        error("Python module 'pdfrw' was not found",
              'Have you followed the instructions is section 1.1 of the manual?')

    # Catch any errors in the script and output them to the log.
    try:
        
        # Find the |.mmz| file we will read, but retain the original filename
        # in |given_mmz_file|, as we will still need it.
        given_mmz_file = mmz_file
        mmz_file = find_in(mmz_file)
        paranoia_in(mmz_file)
        if not args.keep:
            paranoia_out(mmz_file,
                remark = 'This file is rewritten unless option --keep is given.')
        try:
            mmz = open(mmz_file)
        except FileNotFoundError:
            info(f"File '{given_mmz_file}' does not exist, "
                 f"assuming there's nothing to do")
            endinput()

        # Determine the PDF filename: it is either given via |--pdf|, or
        # constructed from the |.mmz| filename.
        pdf_file = find_in(Path(args.pdf)
                           if args.pdf else given_mmz_file.with_suffix('.pdf'))
        paranoia_in(pdf_file)
        if args.prune:
            paranoia_out(pdf_file,
                remark = 'I would have to rewrite this file '
                         'because option --prune was given.')

        # Various initializations.
        
        re_prefix = re.compile(r'\\mmzPrefix *{(?P<prefix>.*?)}')
        re_split_prefix = re.compile(r'(?P<dir_prefix>.*/)?(?P<name_prefix>.*?)')
        re_newextern = re.compile(
            r'\\mmzNewExtern *{(?P<extern_path>.*?)}{(?P<page_n>[0-9]+)}'
            r'{(?P<expected_width>[0-9.]*)pt}{(?P<expected_height>[0-9.]*)pt}')
        re_extern_path = re.compile(
            r'(?P<dir_prefix>.*/)?(?P<name_prefix>.*?)'
            r'(?P<code_md5sum>[0-9A-F]{32})-'
            r'(?P<context_md5sum>[0-9A-F]{32})(?:-[0-9]+)?.pdf')
        pdf = None
        extern_pages = []
        new_mmz = []
        tolerance = 0.01
        dir_to_make = None
        info(f"Extracting new externs listed in '{mmz_file}' from '{pdf_file}'")
        done_message = "Done (there was nothing to extract)"
        indent = '  '
        texindent = r'\space\space '

        # \paragraph{Process \texttt{.mmz}}

        for line in mmz:
            try:
                if m_p := re_prefix.match(line):
                    # Found |\mmzPrefix|: create the extern directory, but only
                    # later, if an extern file is actually produced.  We parse
                    # the prefix in two steps because we have to unquote the
                    # entire prefix.
                    prefix = unquote(m_p['prefix'])
                    if not (m_sp := re_split_prefix.match(prefix)):
                        warning(f"Cannot parse line {line.strip()}")
                    dir_to_make = m_sp['dir_prefix']
                elif m_ne := re_newextern.match(line):
                    # Found |\mmzNewExtern|: extract the extern page into an
                    # extern file.
                    done_message = "Done"
                    # The extern filename, as specified in |.mmz|:
                    unquoted_extern_path = unquote(m_ne['extern_path'])
                    extern_file = Path(unquoted_extern_path)
                    # We parse the extern filename in a separate step because
                    # we have to unquote the entire path.
                    if not (m_ep := re_extern_path.match(unquoted_extern_path)):
                        warning(f"Cannot parse line {line.strip()}")
                    # The actual extern filename:
                    extern_file_out = find_out(extern_file)
                    paranoia_out(extern_file_out)
                    page_n = int(m_ne['page_n'])-1
                    # Check whether c-memo and cc-memo exist (in any input
                    # directory).
                    c_memo = extern_file.with_name(
                        m_ep['name_prefix'] + m_ep['code_md5sum'] + '.memo')
                    cc_memo = extern_file.with_name(
                        m_ep['name_prefix'] + m_ep['code_md5sum']
                        + '-' + m_ep['context_md5sum'] + '.memo')
                    c_memo_in = find_in(c_memo)
                    cc_memo_in = find_in(cc_memo)
                    if not (access_in(c_memo_in) and access_in(cc_memo_in)) \
                       and not args.force:
                        warning(f"I refuse to extract page {page_n+1} into extern "
                                f"'{extern_file}', because the associated c-memo "
                                f"'{c_memo}' and/or cc-memo '{cc_memo}' "
                                f"does not exist")
                        raise NotExtracted()
                    # Load the PDF.  We only do this now so that we don't load
                    # it if there is nothing to extract.
                    if not pdf:
                        if not access_in(pdf_file):
                            warning(f"Cannot open '{pdf_file}'")
                            endinput()
                        try:
                            # All safe, |paranoia_in| was already called above.
                            pdf = pdfrw.PdfReader(pdf_file)
                        except pdfrw.errors.PdfParseError as err:
                            error(rf"File '{pdf_file}' seems corrupted. Perhaps you "
                                  rf"have to load Memoize earlier in the preamble",
                                  rf"In particular, Memoize must be loaded before "
                                  rf"TikZ library 'fadings' and any package "
                                  rf"deploying it, and in Beamer, load Memoize "
                                  rf"by writing \RequirePackage{{memoize}} before "
                                  rf"\documentclass{{beamer}}. "
                                  rf"This was the error thrown by Python: \n{err}")
                    # Does the page exist?
                    if page_n >= len(pdf.pages):
                        error(rf"I cannot extract page {page_n} from '{pdf_file}', "
                              rf"as it contains only {len(pdf.pages)} page" +
                              ('s' if len(pdf.pages) > 1 else ''), '')
                    # Check whether the page size matches the |.mmz|
                    # expectations.
                    page = pdf.pages[page_n]
                    expected_width_pt = float(m_ne['expected_width'])
                    expected_height_pt = float(m_ne['expected_height'])
                    mb = page['/MediaBox']
                    width_bp = float(mb[2]) - float(mb[0])
                    height_bp = float(mb[3]) - float(mb[1])
                    width_pt = width_bp / 72 * 72.27
                    height_pt = height_bp / 72 * 72.27
                    if (abs(width_pt - expected_width_pt) > tolerance
                            or abs(height_pt - expected_height_pt) > tolerance) \
                            and not args.force:
                        warning(
                            f"I refuse to extract page {page_n+1} from '{pdf_file}' "
                            f"because its size ({width_pt}pt x {height_pt}pt) "
                            f"is not what I expected "
                            f"({expected_width_pt}pt x {expected_height_pt}pt)")
                        raise NotExtracted()
                    # All tests were successful, let's create the extern file.
                    # First, the containing directory, if necessary.
                    if dir_to_make:
                        mkdir(dir_to_make)
                        dir_to_make = None
                    # Now the extern file.  Note that |paranoia_out| was
                    # already called above.
                    info(f"Page {page_n+1} --> {extern_file_out}")
                    extern = pdfrw.PdfWriter(extern_file_out)
                    extern.addpage(page)
                    extern.write()
                    # This page will get pruned.
                    if args.prune:
                        extern_pages.append(page_n)
                    # Comment out this |\mmzNewExtern|.
                    if not args.keep:
                        line = '%' + line
            except NotExtracted:
                pass
            finally:
                if not args.keep:
                    new_mmz.append(line)
        mmz.close()
        indent = ''
        texindent = ''
        info(done_message)

        # Write out the |.mmz| file with |\mmzNewExtern| lines commented
        # out. (All safe, |paranoia_out| was already called above.)
        if not args.keep:
            with open(mmz_file, 'w') as mmz:
                for line in new_mmz:
                    print(line, file = mmz, end = '')

        # Remove the extracted pages from the original PDF. (All safe,
        # |paranoia_out| was already called above.)
        if args.prune and extern_pages:
            pruned_pdf = pdfrw.PdfWriter(pdf_file)
            pruned_pdf.addpages(
                page for n, page in enumerate(pdf.pages) if n not in extern_pages)
            pruned_pdf.write()
            info(f"The following extern pages were pruned out of the PDF: " +
                 ",".join(str(page+1) for page in extern_pages))

        # Report that extraction was successful.
        endinput()

    # Catch any errors in the script and output them to the log.
    except Exception as err:
        error(f'Python error: {err}', traceback.format_exc())
        
# Local Variables:
# fill-column: 79
# after-save-hook: py2dtx
# End:



#!/usr/bin/env python3
#coding=utf8

"""

Arguments passes :
    - fichier image (image.fig ou image.svg)
    - -s fichier subfig (image.subfig)
    - -p chemin du script pour generer les sous-images (svgdepth.py ou figdepth.py)

Sortie standard :
    - makefile pour creer les sous-images (au format .fig ou .svg), et pour les supprimer

"""

from __future__ import print_function
from optparse import OptionParser
import os.path

def main():
    parser = OptionParser(usage='usage: %prog [options] svg file', description='Creates a\
Makefile generating subfigures using figdepth.py or svgdepth.py')
    parser.add_option("-s", "--subfig", dest="subfig", help="subfig file")
    parser.add_option("-p", "--depth", dest="depth", help="full path of depth script")
    (options, args) = parser.parse_args()
    if len(args) < 1:
        parser.error("incorrect number of arguments")
    if not options.subfig:
        parser.error("no subfig file specified")
    if not options.depth:
        parser.error("no depth script specified")

    (root, ext) = os.path.splitext(args[0])
    sf_name = options.subfig
    ds_name = options.depth
    varname = '%s_FIGS' % root.upper()

    subfigs = []
    for line in open(options.subfig, 'r'):
        t = line.find('#') # looking for comments
        if t > -1: line = line[0:t] # remove comments...
        line = line.strip() #remove blank chars
        if line == '': continue
        subfigs.append(line)

    count = 1
    for subfig in subfigs:
        print("%s_%d%s: %s%s %s" % (root, count, ext, root, ext, sf_name))
        print("\t%s %s" % (ds_name, subfig))
        print("")
        count += 1
    print("%s := $(foreach n, " % varname, end='')
    count = 1
    for subfig in subfigs:
        print('%d ' % count, end='')
        count += 1
    print(", %s_$(n)%s)" % (root, ext))
    print("FILES_TO_DISTCLEAN += $(%s)" % varname)
    print("FIGS2CREATE_LIST += $(%s)" % varname)
    print("$(TEMPORAIRE): $(%s)" % varname)

if __name__ == "__main__":
    main()



#!/usr/bin/env python3
#coding=utf8

"""

stdin : the original LaTeX log file
stdout : the output filtered log file

"""

from __future__ import print_function
import optparse
import os.path
import re
import sys
import io

def main():
    parser = optparse.OptionParser()
    (options, args) = parser.parse_args()

    display = 0
    in_display = 0
    start_line = ''
    warnerror_re = re.compile(r"^(LaTeX|Package|Class)( (.*))? (Warning:|Error:)")
    fullbox_re = re.compile(r"^(Underfull|Overfull) \\[hv]box")
    accu = ''
    # PDFLaTeX log file is not really in latin-1 (in T1 more exactly)
    # but all bytes are corrects in latin-1, so python won't stop
    # while parsing log.
    # Without specifying this encoding (ie using default utf-8), we
    # can get decode errors (UnicodeDecodeError: 'utf-8' codec can't decode byte...)
    with io.open(sys.stdin.fileno(),'r',encoding='latin-1') as sin:
        for line in sin:
            if display > 0:
                display -= 1
            if line[0:4].lower() in ('info', 'warn') or line[0:5].lower() == 'error':
                display = 0
            line_groups = warnerror_re.match(line)
            if line_groups:
                start_line = line_groups.group(3)
                if not start_line:
                    start_line = ''
                if line_groups.group(2):
                    start_line = "(" + start_line + ")"
                display = 1
                in_display = 1
            elif (start_line != '') and (line[0:len(start_line)] == start_line):
                display = 1
            elif line == "\n":
                in_display = 0
            elif line[0:4] == 'Chap':
                display = 1
            elif fullbox_re.match(line):
                display = 2
            if display:
                print(accu, end="")
                accu = line
            elif in_display:
                print(accu[0:-1], end="")
                accu = line

if __name__ == "__main__":
    main()




#!/usr/bin/env python3
#coding=utf8

import sys
import xml.parsers.expat

layers = []
for arg in sys.argv:
  layers.append(arg)

parser = xml.parsers.expat.ParserCreate()
class XmlParser(object):
    def __init__(self, layers):
        self.state_stack = [True]
        self.last_state = True
        self.layers = layers
    def XmlDeclHandler(self, version, encoding, standalone):
        sys.stdout.write("<?xml version='%s' encoding='%s'?>\n" % (version, encoding))
    def StartDoctypeDeclHandler(self, doctypeName, systemId, publicId, has_internal_subset):
        if publicId != None: sys.stdout.write("<!DOCTYPE %s PUBLIC \"%s\" \"%s\">\n" %\
            (doctypeName, publicId, systemId))
        else: sys.stdout.write("<!DOCTYPE %s \"%s\">\n" % (doctypeName, systemId))
    def StartElementHandler(self, name, attributes):
        if name.lower() == 'g':
            r = self.last_state and ('id' not in attributes or \
                attributes['id'] in self.layers)
            self.last_state = r
            self.state_stack.append(r)
        if not self.last_state: return
        s = ""
        for k, v in attributes.items(): s += ' %s="%s"' % (k, v)
        sys.stdout.write("<%s%s>" % (name, s))
    def EndElementHandler(self, name):
        r = self.last_state
        if name.lower() == 'g':
            self.state_stack = self.state_stack[0:-1]
            self.last_state = self.state_stack[-1]
        if not r: return
        sys.stdout.write("</%s>" % (name))
    def CharacterDataHandler(self, data):
        if not self.last_state: return
        sys.stdout.write(data)

my_parser = XmlParser(layers)

parser.XmlDeclHandler = my_parser.XmlDeclHandler
parser.StartDoctypeDeclHandler = my_parser.StartDoctypeDeclHandler
parser.StartElementHandler = my_parser.StartElementHandler
parser.EndElementHandler = my_parser.EndElementHandler
parser.CharacterDataHandler = my_parser.CharacterDataHandler

for line in sys.stdin:
    parser.Parse(line, False)
parser.Parse('', True)




#!/usr/bin/env python

# This file is a part of Memoize, a TeX package for externalization of
# graphics and memoization of compilation results in general, available at
# https://ctan.org/pkg/memoize and https://github.com/sasozivanovic/memoize.
#
# Copyright (c) 2020- Saso Zivanovic <saso.zivanovic@guest.arnes.si>
#
# This work may be distributed and/or modified under the conditions of the
# LaTeX Project Public License, either version 1.3c of this license or (at
# your option) any later version.  The latest version of this license is in
# https://www.latex-project.org/lppl.txt and version 1.3c or later is part of
# all distributions of LaTeX version 2008 or later.
#
# This work has the LPPL maintenance status `maintained'.
# The Current Maintainer of this work is Saso Zivanovic.
# 
# The files belonging to this work and covered by LPPL are listed in
# <texmf>/doc/generic/memoize/FILES.

__version__ = '2024/12/02 v1.4.1'

import argparse, re, sys, pathlib, os

parser = argparse.ArgumentParser(
    description="Remove (stale) memo and extern files.",
    epilog = "For details, see the man page or the Memoize documentation "
             "(https://ctan.org/pkg/memoize)."
)
parser.add_argument('--yes', '-y', action = 'store_true',
                    help = 'Do not ask for confirmation.')
parser.add_argument('--all', '-a', action = 'store_true',
                    help = 'Remove *all* memos and externs.')
parser.add_argument('--quiet', '-q', action = 'store_true')
parser.add_argument('--prefix', '-p', action = 'append', default = [],
    help = 'A path prefix to clean; this option can be specified multiple times.')
parser.add_argument('mmz', nargs= '*', help='.mmz record files')
parser.add_argument('--version', '-V', action = 'version',
                    version = f"%(prog)s of Memoize " + __version__)
args = parser.parse_args()

re_prefix = re.compile(r'\\mmzPrefix *{(.*?)}')
re_memo = re.compile(r'%? *\\mmz(?:New|Used)(?:CC?Memo|Extern) *{(.*?)}')
re_endinput = re.compile(r' *\\endinput *$')

prefixes = set(pathlib.Path(prefix).resolve() for prefix in args.prefix)
keep = set()

# We loop through the given .mmz files, adding prefixes to whatever manually
# specified by the user, and collecting the files to keep.
for mmz_fn in args.mmz:
    mmz = pathlib.Path(mmz_fn)
    mmz_parent = mmz.parent.resolve()
    try:
        with open(mmz) as mmz_fh:
            prefix = ''
            endinput = False
            empty = None
            for line in mmz_fh:
                line = line.strip()
                
                if not line:
                    pass
                
                elif endinput:
                    raise RuntimeError(
                        rf'Bailing out, '
                        rf'\endinput is not the last line of file {mmz_fn}.')
                
                elif m := re_prefix.match(line):
                    prefix = m[1]
                    prefixes.add( (mmz_parent/prefix).resolve() )
                    if empty is None:
                        empty = True

                elif m := re_memo.match(line):
                    if not prefix:
                        raise RuntimeError(
                            f'Bailing out, no prefix announced before file "{m[1]}".')
                    if not m[1].startswith(prefix):
                        raise RuntimeError(
                            f'Bailing out, prefix of file "{m[1]}" does not match '
                            f'the last announced prefix ({prefix}).')
                    keep.add((mmz_parent / m[1]))
                    empty = False

                elif re_endinput.match(line):
                    endinput = True
                    continue

                else:
                    raise RuntimeError(fr"Bailing out, "
                        fr"file {mmz_fn} contains an unrecognized line: {line}")
            
        if empty and not args.all:
            raise RuntimeError(fr'Bailing out, file {mmz_fn} is empty.')

        if not endinput and empty is not None and not args.all:
            raise RuntimeError(
                fr'Bailing out, file {mmz_fn} does not end with \endinput; '
                fr'this could mean that the compilation did not finish properly. '
                fr'You can only clean with --all.'
            )
        
    # It is not an error if the file doesn't exist.
    # Otherwise, cleaning from scripts would be cumbersome.
    except FileNotFoundError:
        pass

tbdeleted = []
def populate_tbdeleted(folder, basename_prefix):
    re_aux = re.compile(
        re.escape(basename_prefix) + 
        r'[0-9A-F]{32}(?:-[0-9A-F]{32})?'
        r'(?:-[0-9]+)?(?:\.memo|(?:-[0-9]+)?\.pdf|\.log)$')
    try:
        for f in folder.iterdir():
            if re_aux.match(f.name) and (args.all or f not in keep):
                tbdeleted.append(f)
    except FileNotFoundError:
        pass

for prefix in prefixes:
    # "prefix" is interpreted both as a directory (if it exists) and a basename prefix.
    if prefix.is_dir():
        populate_tbdeleted(prefix, '')
    populate_tbdeleted(prefix.parent, prefix.name)

allowed_dirs = [pathlib.Path().absolute()] # todo: output directory
deletion_not_allowed = [f for f in tbdeleted if not f.is_relative_to(*allowed_dirs)]
if deletion_not_allowed:
    raise RuntimeError("Bailing out, "
        "I was asked to delete these files outside the current directory:\n" +
        "\n".join(str(f) for f in deletion_not_allowed))
        
_cwd_absolute = pathlib.Path().absolute()
def relativize(path):
    try:
        return path.relative_to(_cwd_absolute)
    except ValueError:
        return path

if tbdeleted:
    tbdeleted.sort()
    if not args.yes:
        print('I will delete the following files:')
        for f in tbdeleted:
            print(relativize(f))
        print("Proceed (y/n)? ")
        a = input()
    if args.yes or a == 'y' or a == 'yes':
        for f in tbdeleted:
            if not args.quiet:
                print("Deleting", relativize(f))
            try:
                f.unlink()
            except FileNotFoundError:
                print(f"Cannot delete {f}")
    else:
        print("Bailing out.")
elif not args.quiet:
    print('Nothing to do, the directory seems clean.')

# Local Variables:
# fill-column: 79
# after-save-hook: py2dtx
# End:



"""Module to plot using Pgfplots.

This module provides a means to create and display a graph very quickly.  

In this code, the program used to display the created PDF is 'xpdf'.  Change
it to your favorite PDF reader, such as Acrobat Reader (called acroread or
something similar)

The code used to generate the graph is printed in the command line. Edit
your graph iteratively, and when you are satisfied with the graph, copy and
paste the relevant part to your TEX file. 

This module requires the numpy module.

For example of usage, see the executable part at the bottom.


ATTENTION: this file has been provided by 3rd party users in the hope that it
may be useful. However, it is not maintained by the pgfplots team as such. Use at your own risk.

See also a related (improved) pgfplots code generation for python on https://github.com/olivierverdier/pygfplots

"""
import numpy as np
import subprocess
import os
GRAPH_N = 0

class Pgf:
    def __init__(z, xlabel='', ylabel=''):
        """Initialize and provide axis labels."""
        z.buf = []
        z.options = []
        z.opt('xlabel={{{0}}}'.format(xlabel))
        z.opt('ylabel={{{0}}}'.format(ylabel))
        z.legend = []
    def opt(z, *args):
        """Write arguments to the AXIS environment."""
        for arg in args:
            z.options.append(arg)
    def plot(z, x, y, legend=None, *args):
        """Plot the data contained in the vectors x and y.

        Options to the \addplot command can be provided in *args.
        """
        coor = ''.join(['({0}, {1})'. format(u, v) for u, v in zip(x,y)])
        z.buf.append('\\addplot{0} coordinates {{{1}}};\n'.format(
                ('[' + ', '.join(args) + ']') if len(args) else '' ,coor))
        if legend is not None:
            z.legend.append(legend)
    def save(z, graph_n=None):
        """Generate graph.
        
        If graph_n is None or a number, the graph in a file beginning with
        zzz.  This file is meant to be temporary.  If graph_n is a string,
        that string is used as the file name.
        """
        if type(graph_n) is str:
            file_name = graph_n
        else:
            if graph_n is None:
                global GRAPH_N
                graph_n = GRAPH_N
                GRAPH_N += 1
            elif type(graph_n) is not int:
                raise Error('graph_n should be a string or an integer')
            file_name = 'zzz{0}'.format(graph_n)
        with open(file_name + '.tex', 'w') as f:
            b = []
            b.append('\\documentclass{article}\n')
            b.append('\\usepackage{pgfplots}\n')
            b.append('\\begin{document}\n')
            b.append('\\begin{tikzpicture}')
            b.append('\\begin{axis}[\n')
            b.append('{0}]'.format(',\n'.join(z.options)))
            b.extend(z.buf)
            if z.legend:
                b.append('\\legend{{' + '}, {'.join(z.legend) + '}}\n')
            b.append('\\end{axis}\n')
            b.append('\\end{tikzpicture}\n')
            b.append('\\end{document}')
            f.writelines(b)
        print(''.join(b))
        os.system('pdflatex {0}.tex'.format(file_name))
        os.remove(file_name + '.aux')
        os.remove(file_name + '.log')
        subprocess.Popen(['xpdf',  '{0}.pdf'.format(file_name)])
if __name__ == '__main__':
    """Example of usage."""
    x = np.linspace(0, 2*np.pi)
    p = Pgf('time', 'Voltage')
    p.opt('ybar')
    p.plot(x, np.sin(x), 'sin')
    p.plot(x, np.cos(x), 'cos')
    p.save()#'graph_test_pgf_1')
